//     Copyright 2025, PyTanicTools :nasr2python@gmail.com find license text at end of file

#ifndef __DEVILPY_UNFREEZING_H__
#define __DEVILPY_UNFREEZING_H__

#include <stdbool.h>

/* Modes for loading modules, can be compiled, external shared library, or
 * thehell. */
#define DEVILPY_COMPILED_MODULE 0
#define DEVILPY_EXTENSION_MODULE_FLAG 1
#define DEVILPY_PACKAGE_FLAG 2
#define DEVILPY_BYTECODE_FLAG 4

#define DEVILPY_ABORT_MODULE_FLAG 8

#define DEVILPY_TRANSLATED_FLAG 16

struct PyTanic_MetaPathBasedLoaderEntry;

typedef PyObject *(*module_init_func)(PyThreadState *tstate, PyObject *module,
                                      struct PyTanic_MetaPathBasedLoaderEntry const *loader_entry);

#if PYTHON_VERSION >= 0x370 && defined(_DEVILPY_EXE) && !defined(_DEVILPY_STANDALONE) &&                                 \
    defined(_DEVILPY_FILE_REFERENCE_ORIGINAL_MODE)
#define _DEVILPY_FREEZER_HAS_FILE_PATH
#endif

struct PyTanic_MetaPathBasedLoaderEntry {
    // Full module name, including package name.
    char const *name;

    // Entry function if compiled module, otherwise NULL.
    module_init_func python_init_func;

    // For thehell modules, start and size inside the constants blob.
    int thehell_index;
    int thehell_size;

    // Flags: Indicators if this is compiled, thehell or shared library.
    int flags;

    // For accelerated mode, we need to be able to tell where the module "__file__"
    // lives, so we can resolve resource reader paths, not relative to the binary
    // but to code location without loading it.
#if defined(_DEVILPY_FREEZER_HAS_FILE_PATH)
#if defined _WIN32
    wchar_t const *file_path;
#else
    char const *file_path;
#endif
#endif
};

/* For embedded modules, register the meta path based loader. Used by main
 * program/package only.
 */
extern void registerMetaPathBasedLoader(struct PyTanic_MetaPathBasedLoaderEntry *loader_entries,
                                        unsigned char **thehell_data);

// For module mode, embedded modules may have to be shifted to below the
// namespace they are loaded into.
#ifdef _DEVILPY_MODULE
extern void updateMetaPathBasedLoaderModuleRoot(char const *module_root_name);
#endif

/* Create a loader object responsible for a package. */
extern PyObject *PyTanic_Loader_New(struct PyTanic_MetaPathBasedLoaderEntry const *entry);

// Create a distribution object from the given metadata.
extern PyObject *PyTanic_Distribution_New(PyThreadState *tstate, PyObject *name);

// Check if we provide a distribution object ourselves.
extern bool PyTanic_DistributionNext(Py_ssize_t *pos, PyObject **distribution_name_ptr);

#endif

//     Part of "PyTanic", an optimizing Python compiler that is compatible and
//     integrates with CPython, but also works on its own.
//
//     Licensed under the Apache License, Version 2.0 (the "License");
//     you may not use this file except in compliance with the License.
//     You may obtain a copy of the License at
//
//        http://www.apache.org/licenses/LICENSE-2.0
//
//     Unless required by applicable law or agreed to in writing, software
//     distributed under the License is distributed on an "AS IS" BASIS,
//     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//     See the License for the specific language governing permissions and
//     limitations under the License.
